package cz.upol.jj2.lab06;

import java.awt.FlowLayout;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.SwingWorker;
/**
 * (Jedno z) Reseni problemu se slozitymi vypocty v GUI aplikacich.
 */
public class SwingWorkerTutorial extends JFrame {

	private JButton 	btnStart;
	private JButton 	btnStop;
	private JTextField 	txtInput;
	private JLabel 		lblOutput;
	
	private SwingWorker<Integer, Void> sw;
	
	public SwingWorkerTutorial() {
		super();
		
		/////////// inicializace grafickeho rozhrani ///////////
		
		this.setTitle("Demo");
		this.setLayout(new FlowLayout());
		this.setDefaultCloseOperation(EXIT_ON_CLOSE);
		this.setSize(300, 70);
		
		txtInput = new JTextField("30", 5);
		lblOutput = new JLabel("Vysledek:");

		btnStart = new JButton("start");
		btnStop = new JButton("stop");
		btnStop.setEnabled(false);
		
		this.add(txtInput);
		this.add(lblOutput);
		this.add(btnStart);
		this.add(btnStop);
		
		///////// konec inicializace grafickeho rozhrani ///////////
		
		//zahajeni vypoctu
		btnStart.addActionListener(e -> {
			final int arg = Integer.parseInt(txtInput.getText()); // nacte vstup
			displayWait();                            			  // upravi UI
			sw = new FibWorker(arg);                                 // vytvori worker
			sw.execute();	                                      // zacne provadet vypocet
		});

		//preruseni vypoctu
		btnStop.addActionListener(e -> sw.cancel(true));		// uz mame komu rict, aby prerusil vypocet
	}
	
	/**
	 * Upravi graficke rozhrani tak, aby signalizovalo, ze probiha vypocet
	 */
	public void displayWait() {
		lblOutput.setText("Wait!");
		btnStart.setEnabled(false);
		btnStop.setEnabled(true);
	}
	
	/**
	 * Zobrazi vysledek vypoctu
	 * @param value -- vysledna hodnota 
	 */
	public void displayResult(int value) {
		lblOutput.setText(Integer.toString(value));
		btnStart.setEnabled(true);
		btnStop.setEnabled(false);
	}
	
	/**
	 * Zobrazi, ze vypocet byl prerusené
	 */
	public void displayCancel() {
		lblOutput.setText("Cancelled");
		btnStart.setEnabled(true);
		btnStop.setEnabled(false);
	}
	

	
	/**
	 * Vypocte fibonnaciho cislo, slouzi k demonstraci narocneho vypoctu
	 */
	public static int fib(SwingWorker<Integer, Void> w, int n) {
		if (w.isCancelled()) return -1;
		if (n == 0) return 0;
		if (n == 1) return 1;
		return fib(w, n - 1) + fib(w, n - 2);
	}

	/**
	 * Trida, ktera bude na pozadani provadet deletrvajici vypocet.
	 * V tomto pripade zavola funkci fib() s argumentem arg.
	 *
	 * Vysledek vypoctu je predan uzivatelskemu rozhrani pomoci funkci displayResult()
	 * nebo displayCancel() v pripade, ze vypocet byl prerusen.
	 */
	private class FibWorker extends SwingWorker<Integer, Void> {

		private int arg;

		public FibWorker(int arg) {
			super();
			this.arg = arg;
		}

		@Override
		protected Integer doInBackground() throws Exception {
			// provadi narocny vypocet; tato funkce (pripadne funkce vnorene)
			// NESMI PRIMO pracovat s uzivatelskym rozhranim
			return fib(this, arg);
		}



		@Override
		protected void done() {
			try {
				if (this.isCancelled()) displayCancel();
				else displayResult(this.get());
			} catch (Exception e) {
				e.printStackTrace();
			}
		}

		//existuje i metoda process, ktera umoznuje
		// signalizovat prubeh vypoctu hlavnimu vlaknu (napr. kolik procent
		// uz je spocitano, kde to dava smysl)
		//o postupu informujeme volanim publish v ramci doInBackground()
		// druhy typovy parametr souvisi prave s nimi
	}
}